"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const core = require("./core.cjs");
const utils = require("./utils.cjs");
const encoding = require("./encoding.cjs");
const lodashEs = require("lodash-es");
const react = require("react");
const reactStore = require("@tanstack/react-store");
const typeBasedOptions = {
  string: {
    ...encoding.stringSerializer
  },
  number: {
    ...encoding.numberSerializer
  },
  boolean: {
    ...encoding.booleanSerializer
  },
  datetime: {
    ...encoding.datetimeSerializer
  },
  date: {
    ...encoding.dateSerializer
  }
};
class InternalBuilder {
  constructor(state) {
    this.state = state;
    this.transformParams = null;
  }
  withSerializer(serializer) {
    this.state = {
      ...this.state,
      ...serializer
    };
    return this;
  }
  validate(validator) {
    this.state = {
      ...this.state,
      ...validator
    };
    return this;
  }
  withDefault(value) {
    this.state = {
      ...this.state,
      defaultValue: value
    };
    return this;
  }
  transform(params2) {
    this.transformParams = params2;
    return this;
  }
}
const createInternalBuilder = (key, value) => {
  const options = typeBasedOptions[key] || {};
  const listOptions = key === "list" ? encoding.createListSerializer(value) : {};
  const defaultSerializers = {
    encode: encoding.defaultEncoder,
    decode: encoding.defaultDecoder
  };
  const state = {
    onError: () => {
    },
    validate: () => true,
    defaultValue: null,
    updateType: "replaceIn",
    ...defaultSerializers,
    ...options,
    ...value || {},
    ...listOptions
  };
  return new InternalBuilder(state);
};
const p = new Proxy({}, {
  get(_target, prop) {
    return (value) => {
      const key = prop;
      return createInternalBuilder(key, value);
    };
  }
});
const defaultSetTransformer = {
  set: ({ set: setValue }) => {
    return setValue;
  }
};
const getWithPrefix = (prefix, prop) => {
  return prefix ? `${prefix}-${prop}` : prop;
};
const buildSingle = ({ builder, params: params2, prop, globalOptions }) => {
  const options = builder.state;
  const transformParams = builder.transformParams;
  const typedOptions = options;
  const { set } = transformParams || defaultSetTransformer;
  return {
    useSet: (options2) => {
      const memoedOptions = utils.useMemoOptions(typedOptions, globalOptions, options2);
      const paramName = getWithPrefix(memoedOptions.prefix, prop);
      const setter = params2.useParamSet(paramName, memoedOptions);
      return react.useMemo(() => {
        return set({
          set: setter
        });
      }, [setter]);
    },
    use: (options2) => {
      const memoedOptions = utils.useMemoOptions(typedOptions, globalOptions, options2);
      const paramName = getWithPrefix(memoedOptions.prefix, prop);
      const res = params2.useParamGet(paramName, memoedOptions);
      const setter = params2.useParamSet(paramName, memoedOptions);
      const setterTransformed = react.useMemo(() => {
        return set({
          set: setter
        });
      }, [setter]);
      return [
        res,
        setterTransformed
      ];
    }
  };
};
let params = null;
const init = lodashEs.once(() => {
  params = core.createParams();
});
function create(schema, globalOptions) {
  init();
  let internalGlobalOptions = globalOptions;
  const proxy = new Proxy({}, {
    get(_target, prop) {
      if (prop === "withOptions") {
        return (globalOptions2) => {
          internalGlobalOptions = {
            ...internalGlobalOptions,
            ...globalOptions2
          };
          return proxy;
        };
      }
      if (prop === "build") {
        return (values) => {
          const params2 = Object.entries(values).map(([key, value]) => {
            const schemaKey = utils.camelCaseToKebab(key);
            const builder2 = schema[schemaKey];
            const { defaultValue, encode } = builder2.state;
            if (value === utils.getValue(defaultValue)) {
              return void 0;
            }
            const encodedValue = encoding.encodeParam(value, encode);
            return `${key}=${encodedValue}`;
          }).filter(Boolean);
          return params2.join("&");
        };
      }
      if (prop === "batch") {
        return (fn) => {
          const { api } = params.getInternals();
          reactStore.batch(() => {
            api.batch(fn);
          });
        };
      }
      const schemaField = utils.camelCaseToKebab(prop);
      const builder = schema[schemaField];
      return buildSingle({
        builder,
        params,
        prop,
        globalOptions: internalGlobalOptions
      });
    }
  });
  return proxy;
}
function createSingle(prop, definition) {
  init();
  const builder = definition;
  return buildSingle({
    builder,
    params,
    prop
  });
}
exports.create = create;
exports.createSingle = createSingle;
exports.p = p;
//# sourceMappingURL=params.cjs.map
