import { useCallback, useMemo } from "react";
import { useStore, Store } from "@tanstack/react-store";
import { isEqual } from "lodash-es";
import { useSmartValue, isClient, isParamsTransition, getValue, paramsTransitioning } from "./utils.js";
import { encodeParam, decodeParam } from "./encoding.js";
import { useContextApi } from "./use-api.js";
import { withBatch, defaultApi, dummyApi } from "./api.js";
let isInitialized = false;
const createParams = () => {
  let paramsStore = null;
  let api = null;
  const getLatestParams = (api2) => {
    const params = new URLSearchParams(api2.getSearch());
    const paramsMap = {};
    for (const [key, value] of params) {
      paramsMap[key] = value;
    }
    return paramsMap;
  };
  const init = (contextApi, isClient2) => {
    if (isClient2 && isInitialized) {
      return;
    }
    isInitialized = true;
    api = withBatch(contextApi ?? (isClient2 ? defaultApi() : dummyApi));
    if (isClient2) {
      api.registerListener((state) => {
        if (isParamsTransition(state)) {
          return;
        }
        paramsStore.setState(() => getLatestParams(api));
      });
      paramsStore = new Store(getLatestParams(api));
    }
  };
  const decodeWithDefault = (value, defaultValue, options) => {
    var _a, _b;
    const internalDefaultValue = getValue(defaultValue);
    try {
      if (value === void 0) {
        return internalDefaultValue;
      }
      const decoded = decodeParam(value, options.decode);
      const isValid = options.validate ? options.validate(decoded) : true;
      if (!isValid) {
        (_a = options.onError) == null ? void 0 : _a.call(options, value, decoded);
        return internalDefaultValue;
      }
      return decoded;
    } catch {
      const onErrorValue = (_b = options.onError) == null ? void 0 : _b.call(options, value, void 0);
      return onErrorValue ?? internalDefaultValue;
    }
  };
  const useParamGet = (paramName, options) => {
    const contextApi = useContextApi();
    init(contextApi, isClient);
    const { defaultValue } = options;
    const smartDefaultValue = useSmartValue(defaultValue);
    const value = isClient ? useStore(paramsStore, (s) => s[paramName]) : getLatestParams(api)[paramName];
    const decodedValue = useMemo(() => decodeWithDefault(value, smartDefaultValue, options), [value, smartDefaultValue, options]);
    return useSmartValue(decodedValue);
  };
  const useParamSet = (paramName, options) => {
    const contextApi = useContextApi();
    init(contextApi, isClient);
    const { defaultValue } = options;
    const smartDefaultValue = useSmartValue(defaultValue);
    return useCallback((value, state) => {
      if (!isClient) {
        throw new Error("Cannot use react params on the server");
      }
      const { updateType = "replaceIn", encode } = options;
      const currentValue = paramsStore.state[paramName];
      const internalValue = typeof value === "function" ? value(decodeWithDefault(currentValue, smartDefaultValue, options)) : value;
      const newValue = encodeParam(internalValue, encode);
      if (currentValue === newValue || currentValue === void 0 && isEqual(smartDefaultValue, value)) {
        return;
      }
      paramsStore.setState((s) => {
        return { ...s, [paramName]: newValue };
      });
      const search = api.getSearch();
      const clearAll = !updateType.endsWith("In");
      const isReplace = updateType.startsWith("replace");
      const searchParams = !clearAll ? new URLSearchParams(search) : new URLSearchParams();
      if (!isEqual(smartDefaultValue, value)) {
        searchParams.set(paramName, newValue);
      } else {
        searchParams.delete(paramName);
      }
      const newHref = `?${searchParams.toString()}`;
      const internalState = { [paramsTransitioning]: true, ...state };
      if (isReplace) {
        api.replaceState(newHref, internalState);
      } else {
        api.pushState(newHref, internalState);
      }
    }, [paramName, smartDefaultValue, options]);
  };
  const useParam = (paramName, options) => {
    const value = useParamGet(paramName, options);
    const setter = useParamSet(paramName, options);
    return [value, setter];
  };
  return {
    useParamGet,
    useParamSet,
    useParam,
    getInternals: () => {
      return {
        paramsStore,
        api
      };
    }
  };
};
export {
  createParams
};
//# sourceMappingURL=core.js.map
